module ActiveState
  module Model
    module ClassMethods
      attr_accessor :initial_state

      ##
      # Creates scopes for the given states in this form:
      # scope :my_state, -> { where state_name: MyState.name }
      #
      # Also accepts pairs of scope_name: StateClass pairs:
      # scope :scope_name, -> { where state_name: StateClass.name }
      def scope_for_state(*states, **named)
        states.each do |state|
          named[state.name.demodulize.underscore] = state
        end
        named.each do |k, v|
          scope k, -> { where state_name: v.name }
        end
      end
    end

    attr_accessor :state

    def self.included(mod)
      mod.extend ClassMethods
      mod.validates_with StateValidator
      mod.after_initialize :create_state_object
      mod.before_save :synchronize_state_column
    end

    private

    def create_state_object
      state_class =
        if state_name.nil?
          self.class.initial_state
        else
          Object.const_get state_name
        end
      @state = state_class.new self
    end

    def synchronize_state_column
      self.state_name = state.class.name
    end
  end
end
